
from cStringIO import StringIO
import mimetypes
import os

from PIL import Image

from django.core.files.uploadedfile import SimpleUploadedFile
from django.db import models

class ImageMixin(models.Model):
    """
    Provides a model class with a `image` and `thumbnail` fields.
    You can customize the size of the images by changing the
    IMAGE_SIZE and THUMBNAIL_SIZE properties on the destination model.
    """
    IMAGE_SIZE = (540, 900)
    THUMBNAIL_SIZE = (180, 300)

    image = models.ImageField(
        upload_to='images',
        width_field='image_width',
        height_field='image_height',
        )
    image_url = models.URLField(editable=False)
    image_width = models.IntegerField(auto_created=True)
    image_height = models.IntegerField(auto_created=True)

    thumbnail = models.ImageField(
        upload_to='thumbnails',
        width_field='thumbnail_width',
        height_field='thumbnail_height',
        )
    thumbnail_url = models.URLField(editable=False)
    thumbnail_width = models.IntegerField(auto_created=True)
    thumbnail_height = models.IntegerField(auto_created=True)

    def _resize_images(self):
        """
        Resizes images to have the sizes defined by:
          IMAGE_SIZE
          THUMBNAIL_SIZE
        """
        name, extension = os.path.splitext(self.image.name)
        image = Image.open(self.image)
        if image.mode not in ('L', 'RGB'):
            image = image.convert('RGB')
        thumbnail = image.copy()

        # resize images maintaining aspect ratios
        image.thumbnail(self.IMAGE_SIZE, Image.ANTIALIAS)
        thumbnail.thumbnail(self.THUMBNAIL_SIZE, Image.ANTIALIAS)

        # put resized images on model
        self.image = self._get_simple_uploaded_file(image, name, extension)
        self.thumbnail = self._get_simple_uploaded_file(thumbnail, name, extension)

    def _get_simple_uploaded_file(self, image, name, extension):
        """
        :param image:
            a python PIL ``Image`` instance.

        :param name:
            The file name without the extension.

        :param extension:
            The file extension.

        :returns:
            A django ``SimpleUploadedFile`` instance ready to be saved.
        """
        file_name = name + extension
        mimetype, encoding = mimetypes.guess_type(file_name)
        content_type = mimetype or 'image/png'

        temp_handle = StringIO()
        image.save(temp_handle, self._get_pil_format(extension))
        temp_handle.seek(0) # rewind the file

        suf = SimpleUploadedFile(
            file_name,
            temp_handle.read(),
            content_type=content_type,
            )
        return suf

    def _get_pil_format(self, extension):
        """
        :param extension:
            The file name extension (.png, .jpg, etc...)

        :returns:
            The file format PIL needs from the file extension.
            Eg. PNG or JPEG
        """
        return Image.EXTENSION[extension.lower()]

    def delete(self, using=None):
        """
        When deleting the model, make sure to delete the images
        so they're not orphaned in their storage facility.
        """
        self.image.delete(save=False)
        self.thumbnail.delete(save=False)
        super(ImageMixin, self).delete(using=using)

    def save(self, *args, **kwargs):
        """
        Resize the Images, save the model, then update the image urls
        so they don't need to be re-calculated every time they're used.
        """
        if self.image and not self.image._committed:
            self._resize_images()
        super(ImageMixin, self).save(*args, **kwargs)

    class Meta(object):
        abstract = True




from django.contrib import admin

from image_helper.fields import SizedImageField
from image_helper.widgets import AdminImagePreviewWidget

from sample.models import TestModel

class TestModelAdmin(admin.ModelAdmin):
    list_display = ['preview_thumbnail', 'name']

    def preview_thumbnail(self, obj):
        return '<img src="{}" />'.format(obj.image.thumbnail.url())
    preview_thumbnail.allow_tags = True


    formfield_overrides = {
        SizedImageField: {'widget': AdminImagePreviewWidget},
    }

admin.site.register(TestModel, TestModelAdmin)




import os
from cStringIO import StringIO
import mimetypes

from django.db.models.fields.files import ImageField
from django.core.files.uploadedfile import SimpleUploadedFile
from django.db.models import signals

from PIL import Image

# todo: Add 'delete_with_model' option that will delete thumbnail and image when model is deleted.

class ThumbnailField(object):
    """
    Instances of this class will be used to access data of the
    generated thumbnails
    """

    def __init__(self, name, storage):
        """
        Uses same storage as the parent field
        """
        self.name = name
        self.storage = storage

    def path(self):
        return self.storage.path(self.name)

    def url(self):
        return self.storage.url(self.name)

    def size(self):
        return self.storage.size(self.name)


class SizedImageField(ImageField):
    """
    An Image field that allows auto resizing auto creation of thumbnails.
    """

    def __init__(self, verbose_name=None, name=None, width_field=None,
                       height_field=None, size=None, thumbnail_size=None, **kwargs):
        """
        Added fields:
            - size: a tuple containing width and height to resize image, and
                an optional boolean setting if is wanted forcing that size
                (None for not resizing).
            - thumbnail_size: a tuple with same values than `size' (None for
                not creating a thumbnail

        Example: (640, 480, True) -> Will resize image to a width of 640px and
            a height of 480px. File will be cut if necessary for forcing
            the image to have the desired size
        """
        self.size = self._get_resize_options(size)
        self.thumbnail_size = self._get_resize_options(thumbnail_size)

        super(SizedImageField, self).__init__(verbose_name, name, width_field, height_field, **kwargs)

    def _get_resize_options(self, dimensions):
        """
        :param dimensions:
            A tuple of (width, height, force_size).
            'force_size' can be left off and will default to False.
        """
        if dimensions and isinstance(dimensions, (tuple, list)):
            if len(dimensions) < 3:
                dimensions = tuple(dimensions) + (False,)
            return dimensions

    def contribute_to_class(self, cls, name):
        """
        Makes sure thumbnail gets set when image field initialized.
        """
        super(SizedImageField, self).contribute_to_class(cls, name)
        signals.post_init.connect(self._set_thumbnail, sender=cls)

    def pre_save(self, model_instance, add):
        """
        Resizes, commits image to storage, and returns field's value just before saving.
        """
        file = getattr(model_instance, self.attname)
        if file and not file._committed:
            file.file = self._resize_image(model_instance, file)
            file.save(file.name, file, save=False)
        return file

    def _get_thumbnail_filename(self, filename):
        """
        Returns a thumbnail version of the file name.
        """
        name, ext = os.path.splitext(filename)
        return ''.join([name, '-thumbnail', ext])

    def _create_thumbnail(self, model_instance, thumbnail, image_name):
        """
        Because of how we are using the base file name as a reference to the
        thumbnail and not storing it off anywhere, we need to make sure to
        run the name through the storage's `get_available_name` so the name
        doesn't change once we actually save it.
        """
        thumbnail = self._do_resize(thumbnail, self.thumbnail_size)

        full_image_name = self.storage.get_available_name(self.generate_filename(model_instance, image_name))
        thumbnail_filename = self._get_thumbnail_filename(full_image_name)
        thumb = self._get_simple_uploaded_file(thumbnail, thumbnail_filename)
        self.storage.save(thumbnail_filename, thumb)

    def _resize_image(self, model_instance, image_field):
        """"""
        image_name = image_field.name

        image = Image.open(image_field.file)
        if image.mode not in ('L', 'RGB'):
            image = image.convert('RGB')

        if self.size:
            image = self._do_resize(image, self.size)

        if self.thumbnail_size:
            self._create_thumbnail(model_instance, image.copy(), image_name)

        return self._get_simple_uploaded_file(image, image_name)

    def _do_resize(self, img, dimensions):
        width, height, force_size = dimensions
        if force_size:
            img.resize((width, height), Image.ANTIALIAS)
        else:
            img.thumbnail((width, height), Image.ANTIALIAS)
        return img

    def _set_thumbnail(self, instance=None, **kwargs):
        """
        Sets a `thumbnail` attribute on the image field class.
        On thumbnail you can access name, url, path attributes
        """
        image_field = getattr(instance, self.name)
        if image_field:
            filename = self.generate_filename(instance, image_field.name)
            thumbnail_filename = self._get_thumbnail_filename(filename)

            thumbnail_field = ThumbnailField(thumbnail_filename, self.storage)
            setattr(image_field, 'thumbnail', thumbnail_field)

    def _get_simple_uploaded_file(self, image, file_name):
        """
        :param image:
            a python PIL ``Image`` instance.

        :param file_name:
            The file name of the image.

        :returns:
            A django ``SimpleUploadedFile`` instance ready to be saved.
        """
        extension = os.path.splitext(file_name)[1]

        mimetype, encoding = mimetypes.guess_type(file_name)
        content_type = mimetype or 'image/png'

        temp_handle = StringIO()
        image.save(temp_handle, self._get_pil_format(extension))
        temp_handle.seek(0) # rewind the file

        suf = SimpleUploadedFile(
            file_name,
            temp_handle.read(),
            content_type=content_type,
        )
        return suf

    def _get_pil_format(self, extension):
        """
        :param extension:
            The file name extension (.png, .jpg, etc...)

        :returns:
            The file format PIL needs from the file extension.
            Eg. PNG or JPEG
        """
        return Image.EXTENSION[extension.lower()]


try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^image_helper\.fields\.SizedImageField"])
except ImportError:
    pass




import warnings
warnings.warn(
    "This mixin is deprecated. it will be going away soon.", PendingDeprecationWarning
)


class ImageAdminMixin(object):
    """
    Add this Mixin to your model Admin. Then in the list_display,
    declare 'listview_thumbnail' and your image will show.
    """

    def listview_thumbnail(self, obj):
        return "<img src='%s' height='100' />" % obj.thumbnail.url
    listview_thumbnail.allow_tags = True
